<?php
namespace FiloBlu\ExpiredSessionRedirect\Block;

use FiloBlu\ExpiredSessionRedirect\Observer\SetCookie;
use Magento\Customer\Model\Session;
use Magento\Framework\Stdlib\Cookie\CookieMetadataFactory;
use Magento\Framework\Stdlib\Cookie\PhpCookieManager;
use Magento\Framework\View\Element\Template;

class Expire extends Template
{
    /**
     * @var CookieMetadataFactory
     */
    private $cookieMetadataFactory;
    /**
     * @var PhpCookieManager
     */
    private $cookieManager;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var Session
     */
    private $customerSession;

    /**
     * Expire constructor.
     * @param Template\Context $context
     * @param CookieMetadataFactory $cookieMetadataFactory
     * @param PhpCookieManager $cookieManager
     * @param Session $customerSession
     * @param array $data
     */
    public function __construct(
        Template\Context $context,
        CookieMetadataFactory $cookieMetadataFactory,
        PhpCookieManager $cookieManager,
        Session $customerSession,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->cookieMetadataFactory = $cookieMetadataFactory;
        $this->cookieManager = $cookieManager;
        $this->logger =$context->getLogger();
        $this->scopeConfig = $context->getScopeConfig();
        $this->storeManager = $context->getStoreManager();
        $this->customerSession = $customerSession;
    }

    /**
     * @return string
     */
    public function getExpiredSessionLandingUrl()
    {
        try {
            return $this->storeManager->getStore()->getBaseUrl() . $this->scopeConfig->getValue(SetCookie::EXPIRED_SESSION_REDIRECT_URL, \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE);
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
            return "";
        } catch (\Throwable $t) {
            $this->logger->critical($t->getMessage());
            return "";
        }
    }

    /**
     * @return string
     */
    public function getCookieName()
    {
        return SetCookie::COOKIE_NAME;
    }

    /**
     * @return bool
     */
    public function isCookieValid()
    {
        try {
            $actualPage  = $this->storeManager->getStore()->getBaseUrl() . str_replace("/" . $this->storeManager->getStore()->getCode() . "/", "", $this->getRequest()->getRequestUri());

            if ($actualPage == $this->getExpiredSessionLandingUrl()) {
                return true;
            }
            if ($this->cookieManager->getCookie(SetCookie::COOKIE_NAME) !== null) {

                //set cookie to expired
                if ($customer = $this->customerSession->getCustomer()) {
                    if (!$customer->getId()) {
                        $this->cookieManager->deleteCookie(SetCookie::COOKIE_NAME);
                        return false;
                    }
                }
            }
            return true;
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
        } catch (\Throwable $t) {
            $this->logger->critical($t->getMessage());
        }
    }

    /**
     * @return bool
     */
    public function isLoginPage()
    {
        if (stristr($this->getRequest()->getRequestUri(), 'customer/account/login/')) {
            return true;
        }
        return false;
    }
}
